/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/
#define DO_NEXT_PORT	QEvent::User
#define STOP_SCAN 	QEvent::User + 1

// Two methods to add data to fullPortList
// Either a uint or a qstring that can be converted to an uint
void portscan::addToFullPortList(unsigned int i)
{
	if ((0<i) && (i<=65535))
	{
		QString tmpStr;
		fullPortList->append(tmpStr.setNum(i));
	}
	// ignore out of bounds data - the caller should check and report errors
}

void portscan::addToFullPortList(QString & str)
{
bool ok;
unsigned int i = str.toUInt(&ok);
	
	if (ok)
	{
		addToFullPortList(i);
	}
}


void portscan::setProgressCount()
{
    progressBar->setTotalSteps(fullPortList->count());
}

void portscan::clearResultBox()
{
    resultBox->setText("");
}


void portscan::startScan()
{
    //swop the start/stop scan connection and rename the button
    disconnect( goButton, SIGNAL(clicked()),  this, SLOT(startScan()));
    connect(goButton, SIGNAL(clicked()), this,  SLOT(stopScan()));
    goButton->setText("&Stop Scan");
    //Stop the user editing the options
    serverString->setEnabled(false);
    portList->setEnabled(false);
    timeToWait->setEnabled(false);
    // Get the list of ports to scan
    // Empty the fullPortList
    fullPortList->clear();

    // Get the comma separated strings
    QStringList data = QStringList::split(",",portList->text());

    // Iterate through it.
    for (QStringList::Iterator it = data.begin(); it != data.end(); it++)
    {
	// We know the string is nnnn[-mmmm] due to the validator
	QStringList range = QStringList::split("-",*it);
	switch(range.count())
	{
	case 2:
		unsigned int lower, upper;
		bool ok;
		
		lower = range[0].toUInt(&ok);
		if (!ok)
		{
			// non-uint - flag to user and return
			return;
		}
		upper = range[1].toUInt(&ok);
		if (!ok)
		{
			// non-uint - flag to user and return
			return;
		}
		if (upper > 65535)
		{
			// report error to user, and set to 65535
			upper = 65535;
		}
		
		if (lower <= upper)
		{
			for (unsigned int i=lower; i<=upper; i++)
			{
				addToFullPortList(i);
			}
		}
		else
		{
			// ERROR - range is negative
			// Flag to user and ignore
		}
		break;
	case 1:
		// It's a single number
		addToFullPortList(range[0]);
		break;
	default:
		// Something went wrong - flag to user and ignore
	    break;
	}	
    }
       
    // Create the iterator to trawl through the list
    portIterator = fullPortList->begin();
    //Set the progress bar up
    setProgressCount();
    //Output set of ports scanned
    resultBox->setColor(darkGreen);
    QString tmpStr("Start Scan on ");
    tmpStr += serverString->text();
    tmpStr += " for ports: ";
    tmpStr += portList->text();
    resultBox->append(tmpStr);
    //set global variables
    working = true;
    currentPort = 1;
    //Start the ball rolling
    sendSignal(DO_NEXT_PORT);
}


void portscan::init()
{
    //create socket - deleted when portscan is
    mySocket = new QSocket(this,"ConnectorForPorts");
    
    //create timer - deleted when portscan is
    myTimer = new QTimer(this,"WaitForData");
    
    // create port list - deleted when portscan is
    fullPortList = new QStringList();

    //initialise variables
    currentPort = 0;
    working = false;
    
    // set the validator for the Port list
    QRegExp re("([1-9][0-9]{0,4}(-[1-9][0-9]{0,4})?,)*[1-9][0-9]{0,4}(-[1-9][0-9]{0,4})?");
    // validator deleted when portscan is
    QRegExpValidator *vali=new QRegExpValidator(re, this);
    portList->setValidator(vali);
    
    //Connect the signals to the correct slots that aren't done using designer
    connect(goButton, SIGNAL(clicked()), this, SLOT(startScan()));
    connect(mySocket, SIGNAL(error(int)), this, SLOT(noConnection(int)));
    connect(mySocket, SIGNAL(connected()), this, SLOT(isConnection()));
    connect(mySocket, SIGNAL(readyRead()), this, SLOT(isDataToRead()));
    connect(myTimer,SIGNAL(timeout()), this, SLOT(noDataToRead()));
}


void portscan::stopScan()
{
    //if timer is active, stop it
    if (myTimer->isActive())
    {
	myTimer->stop();
    }
    //swop the start/stop scan connection and rename the button
    disconnect( goButton, SIGNAL(clicked()),  this, SLOT(stopScan()));
    connect( goButton, SIGNAL(clicked()),  this, SLOT(startScan()));
    goButton->setText("&Start Scan");
    //Tell the user how it was stopped
    resultBox->setColor(darkGreen);
    if (currentPort <= fullPortList->count())
    {
	resultBox->append("Scan aborted\n");
    }
    else
    {
	resultBox->append("Scan ended\n");
    }
    //Allow the user to edit the options
    serverString->setEnabled(true);
   portList->setEnabled(true);
    timeToWait->setEnabled(true);

    //reset global variables
    currentPort = 0;
    working = false;
 
}


void portscan::tryNextPort()
{
    QString temp;
    bool ok;
    {
	progressBar->setProgress(currentPort);
	if (currentPort >fullPortList->count())
	{
	    sendSignal(STOP_SCAN);
	}
	else
	{
	    mySocket->connectToHost(serverString->text(), (*portIterator).toUInt(&ok));
	}
    }
    currentPort++;
    portIterator++;
}


void portscan::noConnection(int err)
{
    if (working)
    {
    if (QSocket::ErrHostNotFound == err)
    {
	// Host is not valid so print error message and stop scanning
	resultBox->setColor(red);
	resultBox->setBold(true);
	resultBox->append("Error: Host not found\n");
	resultBox->setBold(false);
	sendSignal(STOP_SCAN);
    }
    else
    {
	//Connection refused - not available
	//continue to search
	sendSignal(DO_NEXT_PORT);
    }
    }
}


void portscan::isConnection()
{
    QString temp;
    //Say there is a connection and do the next one
    resultBox->setColor(red);
    resultBox->append(temp.sprintf("TCP/IP port open: %d\n",mySocket->peerPort()));
   // resultBox->append((mySocket->peerAddress()).toString());
    
    //start the single shot timer
    myTimer->start(1000 * timeToWait->value(), true);

}


void portscan::customEvent(QCustomEvent *e)
{
    switch (e->type())
    {
    case DO_NEXT_PORT:
	tryNextPort();
	break;
	
    case STOP_SCAN:
	stopScan();
	break;
	
	default:
	resultBox->setColor(red);
	resultBox->setBold(true);
	resultBox->append("ERROR: Non defined custom event");
	resultBox->setBold(false);
	break;
    }

}

void portscan::sendSignal(int type)
{
    QCustomEvent *ce;
    
    ce = new QCustomEvent(type); //deleted by QT
    qApp->postEvent(this, ce);
    
}


void portscan::isDataToRead()
{
    //make sure the timer doesn't timeout
    myTimer->stop();
    
    //data may not be in simple text lines, so read it as bytes
    QCString s;
    QString *temp;
    Q_ULONG numBytes;
    while ((numBytes = mySocket->bytesAvailable()) > 0)
    {
	s.resize(numBytes);
	if (mySocket->readBlock(s.data(), numBytes) > 0)
	{
	    //We got something so output it to the resultBox
	    temp= new QString(s);
	    resultBox->setColor(darkBlue);
	    resultBox->append(*temp);
	    delete temp;   
	}
    }
    
    //Close the port - we are only interested in the first bit of data sent
    mySocket->close();
    while (mySocket->state() != QSocket::Idle )
    {
	//wait in the loop
    }
    
    //do the next port
    sendSignal(DO_NEXT_PORT);

}


void portscan::noDataToRead()
{
    mySocket->close();
    while (mySocket->state() != QSocket::Idle)
    {
	//do nothing
    }
    
    //do the next port
    sendSignal(DO_NEXT_PORT);

}
